// @remove-file-on-eject
/**
 * Copyright (c) 2015-present, Facebook, Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */
'use strict';

const fs = require('fs');
const chalk = require('chalk');
const tsconfigPathsJest = require('tsconfig-paths-jest');
const paths = require('../../config/paths');

module.exports = (resolve, rootDir, isEjecting) => {
  // Use this instead of `paths.testsSetup` to avoid putting
  // an absolute filename into configuration after ejecting.
  const setupTestsFile = fs.existsSync(paths.testsSetup)
    ? '<rootDir>/jest.setup.js'
    : undefined;

  const tsConfigProd = require(paths.appTsConfig);
  const tsConfigTest = require(paths.appTsTestConfig);
  const tsConfigPathsMaps = tsConfigTest.compilerOptions && tsConfigTest.compilerOptions.paths
    ? tsconfigPathsJest(tsConfigTest)
    : tsConfigProd.compilerOptions && tsConfigProd.compilerOptions.paths
      ? tsconfigPathsJest(tsConfigProd)
      : {};

  // TODO: I don't know if it's safe or not to just use / as path separator
  // in Jest configs. We need help from somebody with Windows to determine this.
  const config = {
    collectCoverageFrom: [
      'src/**/*.{js,jsx,ts,tsx}',
      '!**/*.d.ts'
    ],
    reporters: [
      'default',
      ['jest-junit', {
        suiteName: 'Unit tests',
        output: 'coverage/junit-unit.xml',
        suiteNameTemplate: '{title}',
        titleTemplate: '{title}',
        ancestorSeparator: ' › ',
      }],
    ],
    resolver: resolve('config/jest/pathResolver.js'),
    setupFiles: [resolve('config/polyfills.js')],
    setupTestFrameworkScriptFile: setupTestsFile,
    testMatch: [
      '<rootDir>/src/**/__tests__/**/*.(j|t)s?(x)',
      '<rootDir>/src/**/?(*.)(spec|test).(j|t)s?(x)',
    ],
    testEnvironment: 'jsdom',
    testURL: 'http://localhost',
    transform: {
      '^.+\\.(js|jsx|mjs)$': isEjecting
        ? '<rootDir>/node_modules/babel-jest'
        : resolve('config/jest/babelTransform.js'),
      '^.+\\.tsx?$': resolve('config/jest/typescriptTransform.js'),
      '^.+\\.(le|c)ss$': resolve('config/jest/cssTransform.js'),
      '^(?!.*\\.(js|jsx|mjs|css|less|json|po)$)': resolve('config/jest/fileTransform.js'),
    },
    transformIgnorePatterns: [
      '[/\\\\]node_modules[/\\\\].+\\.(js|jsx|mjs|ts|tsx)$',
    ],
    moduleNameMapper: Object.assign({}, tsConfigPathsMaps, {
      '^react-native$': 'react-native-web',
      '^static': resolve('config/jest/staticMock.js'),
    }),
    moduleFileExtensions: [
      'web.ts',
      'ts',
      'web.tsx',
      'tsx',
      'web.js',
      'js',
      'web.jsx',
      'jsx',
      'json',
      'node',
      'mjs',
    ],
    globals: {
      'ts-jest': {
        skipBabel: true,
        tsConfigFile: paths.appTsTestConfig,
      },
    },
    verbose: true,
  };
  if (rootDir) {
    config.rootDir = rootDir;
  }
  const overrides = Object.assign({}, require(paths.appPackageJson).jest);
  const supportedKeys = [
    'collectCoverageFrom',
    'coverageReporters',
    'coverageThreshold',
    'snapshotSerializers',
    'moduleNameMapper',
  ];
  if (overrides) {
    supportedKeys.forEach(key => {
      if (overrides.hasOwnProperty(key)) {
        config[key] = overrides[key];
        delete overrides[key];
      }
    });
    const unsupportedKeys = Object.keys(overrides);
    if (unsupportedKeys.length) {
      console.error(
        chalk.red(
          'Out of the box, Create React App only supports overriding ' +
          'these Jest options:\n\n' +
          supportedKeys.map(key => chalk.bold('  \u2022 ' + key)).join('\n') +
          '.\n\n' +
          'These options in your package.json Jest configuration ' +
          'are not currently supported by Create React App:\n\n' +
          unsupportedKeys
            .map(key => chalk.bold('  \u2022 ' + key))
            .join('\n')
        )
      );
      process.exit(1);
    }
  }
  return config;
};
